function [] = dicePulseCompress(newConfig,logFile)
% Parameters: newConfig is the name of the script that initializes all required variables
% if no argument is passed "diceProcessConfig.m" is run by default
% This code reads the raw data, parses and processes
% Processsing steps
%   - Open and parse raw dice files
%   - Create or open reference chrips
%   - Low pass filter data and remove and spurious frequencies
%   - pulseCompress - convolve referenc chirp with the data.
%   - save the compressed data
% Tejendra Dhakal
% 02/01/2012
% LDEO, Columbia University
% B`atch processing.  The script now combines files into batches before
% saving them.
% Channels:  The script can process multiple channels at once.
% Metadata Stacking:  Metadata are not averaged.  Instead, all of the
% metadata that go with a single stacked data frame are stored in a 3D
% matrix.  The 3rd dimension is the unstacked frames that were included in
% each stacked frame.
%01/27/2016 - In order to reduce data size, the radar echoes are truncated to
% to a given range in Ice
%03/23/2016 - Code can now pickup processing where it left off - last batch
%will still be reprocessed
try
fprintf('************** dicePulseCompress.m ****************\n')
if nargin <1
    configScript = 'diceProcessConfig.m';
    logFile = 'diceProcessLog.log';
    diary off;
    if exist(logFile,'file')
        warning('dicePulseCompress:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
    end
    diary(logFile);
    localDiary = true;
elseif nargin <2
    configScript = newConfig;
    logFile = 'diceProcessLog.log';
    diary off;
    if exist(logFile,'file')
        warning('dicePulseCompress:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
    end
    diary(logFile);
    localDiary = true;
else
    configScript = newConfig;
    localDiary = false;
end

clear(configScript);
run(configScript);
tic
configParamsCurr = configParams;

fprintf('dicePulseCompress: Starting DICE Pulse Compression..\n')
% Define filter

[z,p,k] = butter(8,bw/fs*1.1,'low');
dicelpf = zp2sos(z,p,k);

% Apply taper to reference signal:
if usetukey
    sigp2ref = sigp2ref.*tukeywin(Nrefp2,taper);
    sigp1ref = sigp1ref.*tukeywin(Nrefp1,taper);
end

% Create complex baseband ref;
sigp1ref =  cos(2*pi*fc*t1).*sigp1ref - 1i*sin(2*pi*fc*t1).*sigp1ref;
sigp2ref =     cos(2*pi*fc*t2).*sigp2ref - 1i*sin(2*pi*fc*t2).*sigp2ref;

if usewin
    if strcmp(wintype,'kaiser')
        sigp2ref = sigp2ref.*kaiser(Nrefp2,winparam);
        sigp1ref = sigp1ref.*kaiser(Nrefp1,winparam);
    elseif strcmp(wintype,'cheb')
        sigp2ref = sigp2ref.*chebwin(Nrefp2,winparam);
        sigp1ref = sigp1ref.*chebwin(Nrefp1,winparam);
    else
        warning('PulseCompress:InvalidWinType',...
            'Parameter "wintype" must be a string equal to "cheb" or "kaiser"\n No window used!')
    end
end

Nt1 = length(sigp1ref) + truncLenp1 -1;
Nt2 = length(sigp2ref) + truncLenp2 -1;

% Zero-pad reference function to same length as data:
sigp1ref=[sigp1ref;zeros(truncLenp1-1,1)];
sigp2ref=[sigp2ref;zeros(truncLenp2-1,1)];

% Fourier Transform Reference Function:
sigp1ref = conj(fft(sigp1ref,Nt1));
sigp2ref = conj(fft(sigp2ref,Nt2));

%% Produce Notch filters to be used
if(usenotchfilter)
    numNotch = numel(notchList);
    notchFilterObj = cell(size(notchList));
    for ff = 1:numNotch
        notchFilterObj{ff} = designfilt('bandstopiir','FilterOrder',numOrder, ...
            'HalfPowerFrequency1', notchList(ff)-nfBW/2,'HalfPowerFrequency2',notchList(ff)+nfBW/2, ...
            'SampleRate',fs);
    end
    clear ff
end

%% Start Processing Here
for ff = 1:numel(uniqueFlightNum)
    FlightNo = uniqueFlight{ff};
    pulseCompressedTime = now;
    
    prodLevel.name = 'level1';
    if(doCompression)
        prodLevel.desc{1} = 'pulseCompressed';
    else
        prodLevel.desc{1} = 'parsed';
    end
    prodLevel.time{1} = datestr(now);
        
    %% Check files and directories:
    rawdatapath = [sensorRootPath,filesep,rawDir,filesep,FlightNo,filesep,rawDataDir,filesep];
    pulseCompressedDataPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,FlightNo,filesep,dataDir,filesep];    
    processLogPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,FlightNo,filesep,logDir,filesep];
    if exist(rawdatapath,'dir')==0
        warning('dicePulseCompress:noRawDir','Raw Data Flight %s Directory does not exists!',FlightNo);
        continue;
    end
    
        %% Check input directory and sort data files
    % Find input files:
    dFileStr = dir([rawdatapath,rawFileKey]);
    numDataFiles = length(dFileStr);
    if numDataFiles==0
        error('dicePulseCompress:NorawFilesInDir','Data Files Not Found!')
    end
    
    % Sort data files by timestamp:
    for ii = 1:numDataFiles
        fileNameFields = strsplit(dFileStr(ii).name,fileFieldSep);
        if numel(fileNameFields) ~= numRawFileFields
            warning('PulseCompress:InvalidFileNameFormat',['File %s does not conform to DICE file format specifications\n',...
                'Please make sure all DICE data files in %s are in format "%s"\n %s will not be processed'],dFileStr(ii).name,rawdatapath,rawFileNameFormat,FlightNo)
            break;
        else
            dFileStr(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[rawFileDateFormat rawFileTimeFormat]));
        end
    end
    if(ii<numDataFiles) % file sorting did not complete - lets move on to the next flight.
        warning('dicePulseCompress:sortingIncomplete','All files canno not be sorted - %s will be skipped',FlightNo);
        continue;
    end
    clear fileNameFields;
    
    [~,sx] = sort([dFileStr.utc]);
    dFileStr = dFileStr(sx);
    fTime = [dFileStr.utc];
    
    
    %% Check for existing output directories and data (if any)
    if ~exist(pulseCompressedDataPath,'dir')
        mkdir(pulseCompressedDataPath)
    else
        dataFlist = dir([pulseCompressedDataPath,'*',FlightNo,'*.mat']);
        if numel(dataFlist)>0
            fprintf('dicePulseCompress: Found existing data files\n');
            pcFlist = dir([pulseCompressedDataPath,'*',configFnameRoot,'*.mat']);
            if numel(pcFlist)>0
                [~,sx] = sort([pcFlist.datenum]);
                pcFlist = pcFlist(sx);
                fprintf('dicePulseCompress: Checking existing configuration data...\n')
                load([pulseCompressedDataPath,pcFlist(end).name],'configParams');
                if isequal(configParamsCurr.pulseCompress,configParams.pulseCompress)
                    if ignoreConfigMatch && overwriteoutput
                        warning('dicePulseCompress:FlightDirExists',['Configuration match ignored\n','All existing files will be deleted!\n',...
                            'If this is not intended, press Ctrl + C to cancel'],FlightNo);
                        
                        fprintf('dicePulseCompress: Files will be deleted in %d seconds',9);
                        for ss = 9:-1:0
                            fprintf('\b\b\b\b\b\b\b\b\b%d seconds',ss);
                            pause(1);
                        end
                        fprintf('\ndicePulseCompress: Deleted!\n')
                        delete([pulseCompressedDataPath,'*',FlightNo,'*.mat']);
                        delete([pulseCompressedDataPath,'*onfig*.mat']);
                    else
                        fprintf(['dicePulseCompress: Current settings are identical to the one %s processed data used.\n\t',...
                            'Existing %s data will not be re-processed.\n\tTo force processing, Remove configuration mat file from\n\t%s\n'],FlightNo,FlightNo,pulseCompressedDataPath);
                    end
                else    % Configurations don't match
                    if overwriteoutput == 1
                        warning('dicePulseCompress:FlightDirExists',['New configuration!, all existing files will be deleted!\n',...
                            'If this is not intended, press Ctrl + C to cancel'],FlightNo);
                        
                        fprintf('dicePulseCompress: Files will be deleted in %d seconds',9);
                        for ss = 9:-1:0
                            fprintf('\b\b\b\b\b\b\b\b\b%d seconds',ss);
                            pause(1);
                        end
                        fprintf('\ndicePulseCompress: Deleted!\n')
                        delete([pulseCompressedDataPath,'*',FlightNo,'*.mat']);
                        delete([pulseCompressedDataPath,'*onfig*.mat']);
                    else
                        warning('dicePulseCompress:FlightDirExists','Flight %s Directory already exists - Flight will not be processed!\n Set "overwriteoutput = 1" to force processing',uniqueFlight{ff});
                        continue;
                    end
                end
            else    %if for any reason there is no configuration file
                if overwriteoutput == 1
                    warning('dicePulseCompress:FlightDirExists',['All existing files will be deleted!\n',...
                        'If this is a mistake, press Ctrl + C to cancel'],FlightNo);
                    
                    fprintf('dicePulseCompress: Files will be deleted in %d seconds',9);
                    for ss = 9:-1:0
                        fprintf('\b\b\b\b\b\b\b\b\b%d seconds',ss);
                        pause(1);
                    end
                    fprintf('\ndicePulseCompress: Deleted!\n')
                    delete([pulseCompressedDataPath,'*',FlightNo,'*.mat']);
                    delete([pulseCompressedDataPath,'*onfig*.mat']);
                else
                    warning('dicePulseCompress:FlightDirExists','Flight %s Directory already exists - Flight will not be processed!\n Set "overwriteoutput = 1" to force processing',uniqueFlight{ff});
                    continue;
                end
            end
        end
    end
    % save a copy of configuration setting in output direcory for future
    % reference
    [~,configFilename,~] = fileparts(configScript);
    copyfile([configFilename,'.mat'],[pulseCompressedDataPath,configFilename,'.mat'])
    fprintf('dicePulseCompress: Processing Flight %s\n',FlightNo)
    
    %% Now lets figure out if there are any existing processed files in the output directory.
    % If there are, processing will resume from there. If any averaging is
    % used, there may be gaps as long as numAvg number of echoes where the
    % processing software picks up.
    startfile = zeros(numel(processChan),1);
    endfile = zeros(numel(processChan),1);
    level1Files = cell(numel(processChan),1);
    if exist('numExcessFrames','var')
        clear('numExcessFrames')
    end
    for chan_num = 1:numel(processChan)
        level1Files{chan_num}=dir([pulseCompressedDataPath,['*',processChanAttbr{processChan(chan_num)},'*.mat']]);
        numProcessedfiles=numel(level1Files{chan_num});
        if(numProcessedfiles >1)
            for ii = 1:numProcessedfiles
                fileNameFields = strsplit(level1Files{chan_num}(ii).name,fileFieldSep);
                try
                    level1Files{chan_num}(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[outFileDateFormat,outFileTimeFormat]));
                catch
                    warning('dicePulseCompress:oldFileNameFormat','Found level1 files in old file name format, files will be renamed to new format.')
                    warning('off','dicePulseCompress:oldFileNameFormat')
                    movefile([pulseCompressedDataPath,level1Files{chan_num}(ii).name],[pulseCompressedDataPath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,fileNameFields{4},fileFieldSep,...
                        fileNameFields{5},fileFieldSep,fileNameFields{3},fileFieldSep,fileNameFields{6}]);
                    level1Files{chan_num}(ii).name = [fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,fileNameFields{4},fileFieldSep,...
                        fileNameFields{5},fileFieldSep,fileNameFields{3},fileFieldSep,fileNameFields{6}];
                    fileNameFields = strsplit(level1Files{chan_num}(ii).name,fileFieldSep);
                    level1Files{chan_num}(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[outFileDateFormat,outFileTimeFormat]));
                end
            end
            [~,sx] = sort([level1Files{chan_num}.utc]);
            level1Files{chan_num} = level1Files{chan_num}(sx);
            
            % Looking for a corrupt file, if found processing will start
            % there
            zeroSizeFile = find([level1Files{chan_num}.bytes] <200,1,'first');
            if ~isempty(zeroSizeFile)
                numProcessedfiles = zeroSizeFile -1;
                 warning('dicePuseCompressed:oldProcessedData', 'Found a corrupt file %s, All files from this point on will be reprocessed',...
                    level1Files{chan_num}(zeroSizeFile).name)
            end
            if numProcessedfiles>1
            load([pulseCompressedDataPath,level1Files{chan_num}(numProcessedfiles-1).name],'numexcessFrames','echotimeutc')
            if exist('numexcessFrames','var')
                lastProcessedExcessFrames(chan_num) = numexcessFrames;
            else
                if(numAvg>1)
                    warning('dicePuseCompressed:oldProcessedData',['This data was processed with v1 of processing code\n',...
                        'picking up processing in the middle of the flight may result in loss of up to %d frames\n'...
                        'If this is not desired, reprocess the entire flight by setting "ignoreConfigMatch" and "overwriteoutput" to 1\n'],numAvg);
                end
            end
            lastProcessedEchoTime(chan_num) = echotimeutc(end);
            end
        end
        % Figure out which input files to process
        % if there are no start or stop times all files in the directory are
        % processed
        if exist('lastProcessedEchoTime','var')
            startTime = datestr(unix2matlabtime(lastProcessedEchoTime(chan_num)),dateTimeFormat);
        end
        if ~exist('startTime','var')
            startTime = datestr(719529,dateTimeFormat);
        end
        if ~exist('endTime','var')
            endTime = datestr(now,dateTimeFormat);
        end
        
        startutc= matlab2unixtime(datenum(startTime,dateTimeFormat));
        sfile = find([dFileStr.utc]<startutc,1,'last');
        endutc=matlab2unixtime(datenum(endTime,dateTimeFormat));
        efile = find([dFileStr.utc]>endutc,1,'first');
        
        if isempty(sfile)
            startfile(chan_num)=1;
        else
            startfile(chan_num)=sfile;
        end
        if isempty(efile)
            endfile(chan_num)=numDataFiles;
        else
            endfile(chan_num)=efile;
        end
    end
    
    % If there are multiple channels than the channel with earliest start
    % file is selectd - same applies to excess frames
    % The idea is to load the last file processed and carry over the excess
    % frames that got left out from that file.
    [startfile,minInd] = min(startfile);
    if exist('lastProcessedExcessFrames','var')
        numExcesFrames = repmat(lastProcessedExcessFrames(minInd),4,1);
        clear lastProcessedExcessFrames; % clear this for next flight
    end
    if exist('lastProcessedEchoTime','var')
        lastProcessedEchoTime = lastProcessedEchoTime(minInd);
        if lastProcessedEchoTime == 0   % one of the channels was never processed
            clear lastProcessedEchoTime;
        end
    end
    endfile = max(endfile);
    if startfile >1
        if(endfile - startfile) < 2
            warning('PulseCompress:InvalidStartEndTime','No files selected for processing! check the raw data directory')
            continue;
        end
    else
        if(endfile - startfile) < 1
            warning('PulseCompress:InvalidStartEndTime','No files selected for processing! check the raw data directory')
            continue;
        end
    end
    
    %% Frame Computations
    % Each frame consists of [p1 data (16bit*datalenp1) + metadata(32B) + p2 data(16bit*datalenp2) + metadata(32B)]
    % Compute number of batches to process:
    % Compute frame size and frame length:
    frameSize = (datalenp1*2 + datalenp2*2+80)*2;
    frameLen = frameSize/2;
    maxNumFrames = ceil(max([dFileStr.bytes])/frameSize)+numAvg+1;        % Maximum data traces in a file (integer)
    
    % Find any file with missing frames
    truncatedFile = mod([dFileStr.bytes],frameSize)>0 | [dFileStr.bytes] == 0;
    missingFile = [false diff([dFileStr.utc])>11];      % This determines if files are contiguous
    %% Check Channels to Process:
    
    % Pre-allocate memory:
    numChannels=length(processChan);
    echotypetmp=cell(numChannels,1);
    ref=cell(numChannels,1);
    Nt=zeros(numChannels,1);
    Nref=zeros(numChannels,1);
    spikeStartInd=zeros(numChannels,size(spikeStartIndp1,2));
    spikeStopInd=zeros(numChannels,size(spikeStopIndp1,2));
    dataind=cell(numChannels,1);
    metadataind=cell(numChannels,1);
    lastTime = zeros(numChannels,1);
    NFFT = zeros(numChannels,1);
    iqDemod = cell(numChannels,1);
    truncFFTLen = zeros(numChannels,1);
    pulsTimeOffset = zeros(numChannels,1);
    % Loop through channels to process:
    for channel=1:numChannels
        % Check channel:
        switch processChan(channel)
            case 1
                % Assign echo type:
                echotypetmp{channel} = [p1Len,'Ch1'];
                % Assign and expand reference function:
                ref{channel} = repmat(sigp1ref,[1,maxNumFrames]);
                iqDemod{channel} = repmat(cos(2*pi*fdemod*tdata(1:truncLenp1))-1i*sin(2*pi*fdemod*tdata(1:truncLenp1)),1,maxNumFrames);
                % Assign lengths:
                Nt(channel) = truncLenp1;      %% length to data for processing
                Nref(channel) = Nrefp1;
                NFFT(channel) = Nt1;
                truncFFTLen(channel) = NFFT(channel);
                pulsTimeOffset(channel) = 0;        %% this accounts for the fact that p2 comes after PRI of p1
                % Assign indices of zero replacement interval:
                spikeStartInd(channel,:)=spikeStartIndp1;
                spikeStopInd(channel,:)=spikeStopIndp1;
                % Assign indices of this channel within the input file:
                dataind{channel} = repmat(logical([zeros(2,1);ones(truncLenp1,1);zeros(datalenp1-truncLenp1,1);zeros(numMetadataSamples,1);zeros(2,1);...
                    zeros(datalenp1 + 20 + (datalenp2 + 20)*2, 1)]),floor(maxNumFrames),1);
                metadataind{channel} = repmat(logical([zeros(2,1);zeros(datalenp1,1);ones(numMetadataSamples,1);zeros(2,1);...
                    zeros(datalenp1 + 20 + (datalenp2 + 20)*2, 1)]),floor(maxNumFrames),1);
            case 2
                % Assign echo type:
                echotypetmp{channel} = [p1Len,'Ch2'];
                % Assign and expand reference function:
                ref{channel} = repmat(sigp1ref,[1,maxNumFrames]);
                iqDemod{channel} = repmat(cos(2*pi*fdemod*tdata(1:truncLenp1))-1i*sin(2*pi*fdemod*tdata(1:truncLenp1)),1,maxNumFrames);
                % Assign lengths:
                Nt(channel) = truncLenp1;
                Nref(channel) = Nrefp1;
                NFFT(channel) = Nt1;
                truncFFTLen(channel) = NFFT(channel);
                pulsTimeOffset(channel) = 0;        %% this accounts for the fact that p2 comes after PRI of p1
                % Assign indices of zero replacement interval:
                spikeStartInd(channel,:)=spikeStartIndp1;
                spikeStopInd(channel,:)=spikeStopIndp1;
                % Assign indices of this channel within the input file:
                dataind{channel} = repmat(logical([zeros(datalenp1 + 20,1);zeros(2,1);...
                    ones(truncLenp1,1);zeros(datalenp1-truncLenp1,1);zeros(numMetadataSamples,1);zeros(2,1);...
                    zeros((datalenp2 + 20)*2, 1)]),floor(maxNumFrames),1);
                metadataind{channel} = repmat(logical([zeros(datalenp1 + 20,1);zeros(2,1);...
                    zeros(datalenp1,1);ones(numMetadataSamples,1);zeros(2,1);...
                    zeros((datalenp2 + 20)*2, 1)]),floor(maxNumFrames),1);
            case 3
                % Assign echo type:
                echotypetmp{channel} = [p2Len,'Ch1'];
                % Assign and expand reference function:
                ref{channel} = repmat(sigp2ref,[1,maxNumFrames]);
                iqDemod{channel} = repmat(cos(2*pi*fdemod*tdata(1:truncLenp2))-1i*sin(2*pi*fdemod*tdata(1:truncLenp2)),1,maxNumFrames);
                % Assign lengths:
                Nt(channel) = truncLenp2;
                Nref(channel) = Nrefp2;
                NFFT(channel) = Nt2;
                truncFFTLen(channel) = NFFT(channel);
                pulsTimeOffset(channel) = PRI;        %% this accounts for the fact that p2 comes after PRI of p1
                % Assign indices of zero replacement interval:
                spikeStartInd(channel,:)=spikeStartIndp2;
                spikeStopInd(channel,:)=spikeStopIndp2;
                % Assign indices of this channel within the input file:
                dataind{channel} = repmat(logical([zeros((datalenp1 + 20)*2,1);zeros(2,1);...
                    ones(truncLenp2,1);zeros(datalenp2-truncLenp2,1);zeros(numMetadataSamples,1);zeros(2,1);...
                    zeros((datalenp2 + 20), 1)]),floor(maxNumFrames),1);
                metadataind{channel} = repmat(logical([zeros((datalenp1 + 20)*2,1);zeros(2,1);...
                    zeros(datalenp2,1);ones(numMetadataSamples,1);zeros(2,1);...
                    zeros((datalenp2 + 20), 1)]),floor(maxNumFrames),1);
            otherwise
                % Assign echo type:
                echotypetmp{channel} = [p2Len,'Ch2'];
                % Assign and expand reference function:
                ref{channel} = repmat(sigp2ref,[1,maxNumFrames]);
                iqDemod{channel} = repmat(cos(2*pi*fdemod*tdata(1:truncLenp2))-1i*sin(2*pi*fdemod*tdata(1:truncLenp2)),1,maxNumFrames);
                % Assign lengths:
                Nt(channel) = truncLenp2;
                Nref(channel) = Nrefp2;
                NFFT(channel) = Nt2;
                truncFFTLen(channel) = NFFT(channel);
                pulsTimeOffset(channel) = PRI;        %% this accounts for the fact that p2 comes after PRI of p1
                % Assign indices of zero replacement interval:
                spikeStartInd(channel,:)=spikeStartIndp2;
                spikeStopInd(channel,:)=spikeStopIndp2;
                % Assign indices of this channel within the input file:
                dataind{channel} = repmat(logical([zeros((datalenp1 + 20)*2 + datalenp2 + 20, 1);...
                    zeros(2,1);ones(truncLenp2,1);zeros(datalenp2-truncLenp2,1);zeros(numMetadataSamples,1);zeros(2,1)]),floor(maxNumFrames),1);
                metadataind{channel} = repmat(logical([zeros((datalenp1 + 20)*2 + datalenp2 + 20, 1);...
                    zeros(2,1);zeros(datalenp2,1);ones(numMetadataSamples,1);zeros(2,1)]),floor(maxNumFrames),1);
        end
    end
    %% Miscellaneous Computations:
    % Conversion factor from bits to volts
    elephantstoVolts = 1.125/4096/numHWStacks;
        
    %% Process Data:
    
    % Pre-allocate memory:
    tmpfile=zeros(max([dFileStr.bytes]/2)+frameSize,1);
    if ~exist('numExcessFrames','var')
        numExcessFrames=zeros(numChannels,1);
    end
    radechotmp=cell(numChannels,1);
    utctmp=cell(numChannels,1);
    metadatatmp=cell(numChannels,1);
    datastack=cell(numChannels,1);
    tmpdata=cell(numChannels,1);
    excessdata=cell(numChannels,1);
    excessmetadata=cell(numChannels,1);
    for channel=1:numChannels
        radechotmp{channel}=zeros(Nt(channel),ceil(maxNumFrames/numAvg)*filesperbatch);
        utctmp{channel}=zeros(1,ceil(maxNumFrames/numAvg)*filesperbatch);
        metadatatmp{channel}=zeros(numMetadataSamples,ceil(maxNumFrames/numAvg)*filesperbatch,numAvg);
        datastack{channel}=zeros(NFFT(channel),ceil(maxNumFrames/numAvg));
        tmpdata{channel}=zeros(NFFT(channel),ceil(maxNumFrames/numAvg));
        excessdata{channel}=zeros(Nt(channel),numAvg);
        excessmetadata{channel}=zeros(numMetadataSamples,numAvg);
    end
    
    % If this is a pick-up processing and there are left over frames lets
    % read the file here
    if numExcessFrames(1)>0
        if ~truncatedFile(startfile) && ~missingFile(startfile)
            numFramesInFile = dFileStr(ii).bytes/frameSize;
            numFramesToStack = numFramesInFile - numExcessFrames(1);
            numStackedFrames = numFramesToStack/numAvg;
            fid1 = fopen([rawdatapath,dFileStr(startfile).name],'r');
            % cleans the fileID when the function exits.
            cleaner = onCleanup(@()fclose(fid1));
            tmpfile(1:numFramesInFile*frameSize/2) = fread(fid1,numFramesInFile*frameSize/2,'int16');
            clear('cleaner');
            dataprestack = reshape(tmpfile(dataind{channel}(1:frameLen*numFramesInFile))*elephantstoVolts,Nt(channel),numFramesInFile);
            if(process_metadata == 1)
                metadataprestack = reshape(tmpfile(metadataind{channel}(1:frameLen*numFramesInFile)),numMetadataSamples,numFramesInFile);
            end
            for channnel = 1:numChannels
                excessdata{channel}(:,1:numExcessFrames(channnel))=dataprestack(1:Nt(channel),numAvg*numStackedFrames+1:end);
                if(process_metadata == 1)
                    excessmetadata{channel}(:,1:numExcessFrames(channnel))=metadataprestack(1:numMetadataSamples,numAvg*numStackedFrames+1:end);
                end
            end
        else
            numExcesFrames(:) = 0;
            clear('lastProcessedEchoTime');
        end
    end
    
    % Compute initial number of batches:
    minfNum=zeros(ceil((endfile-startfile+1)/filesperbatch) + numel(find(missingFile)) + numel(find(truncatedFile)) + 1,1);
    maxfNum = minfNum;
    if(startfile>1) % if this is pick-up processing
        minfNum(1)=startfile+1; % first file has already been loaded above
    else
        minfNum(1)=1; % first file has already been loaded above
    end
    
    maxfNum(1)= minfNum(1);%min(minfNum(1)+filesperbatch-1,numDataFiles);
    
    % New batch starts every filesperbatch files
    % if there is a missing file that is not the first file of the batch a
    % new batch is started
    % if there is a truncated file that is not the first file of the batch
    % a new batch is started
    batchnumber = 1;
    for filenumber = minfNum(1)+1:endfile
        numbatches = batchnumber;
        if missingFile(filenumber)
            if(filenumber ~= minfNum(batchnumber))
                maxfNum(batchnumber) = filenumber-1;   % current batch should end on previous file
                minfNum(batchnumber+1) = filenumber;   % next batch should start at the current file
                maxfNum(batchnumber+1) = minfNum(batchnumber+1);
                batchnumber = batchnumber +1;
            end
        elseif filenumber == (minfNum(batchnumber) + filesperbatch - 1)
                maxfNum(batchnumber) = filenumber;
                minfNum(batchnumber+1) = min(filenumber + 1,endfile);
                maxfNum(batchnumber+1) = minfNum(batchnumber+1);
                batchnumber = batchnumber + 1;
        elseif truncatedFile(filenumber)
            if(filenumber ~= minfNum(batchnumber))
                maxfNum(batchnumber) = filenumber;   % current batch should end on current file
                minfNum(batchnumber+1) = min(filenumber+1,endfile);   % next batch should start at the next file
                maxfNum(batchnumber+1) = minfNum(batchnumber+1);
                batchnumber = batchnumber +1;
            end
        elseif filenumber == endfile
               maxfNum(batchnumber) = filenumber;
        end
    end
    minfNum = minfNum(1:numbatches);
    maxfNum = maxfNum(1:numbatches);
    clearvars batchnumber filenumber;
    
    
    if exist('lastProcessedEchoTime','var')
        lastTime(1:numChannels) = lastProcessedEchoTime - pulsTimeOffset(minInd) + 1/prfPreSar + pulsTimeOffset(1:numChannels);
        clear lastProcessedEchoTime; % clear this for next flight
    else
        lastTime(1:numChannels) = fTime(minfNum(1))+(numAvg-1)*PRI/2 + pulsTimeOffset(1:numChannels);
    end
    % Loop through batches:
    for batch=1:numbatches
        
        % Communicate with human:
        fprintf('dicePulseCompress: Processing batch %d of %d\n',batch,numbatches)
        
        % Reset echo counter:
        numechos = ones(numChannels,1);
        
        % Loop through files:
        for ii = minfNum(batch):maxfNum(batch)
            
            % Check file size:
            if truncatedFile(ii)
                warning('PulseCompress:TruncatedFile','Warning: Skipping Truncated File %s',dFileStr(ii).name)
                if(ii < endfile)
                    lastTime(1:numChannels) = fTime(ii+1)+(numAvg-1)*PRI/2 + pulsTimeOffset(1:numChannels); % reset the start time
                end
                numExcessFrames(:) = 0;     % reset excess frame count
                continue
            end
            
            
            % update the last time if there is a missing file
            if missingFile(ii)
                lastTime(1:numChannels) = fTime(ii)+(numAvg-1)*PRI/2+ pulsTimeOffset(1:numChannels); % uses average time for the pulses that are stacked
                numExcessFrames(:) = 0;     % reset excess frame count
                if(batch >1)
                    warning('PuseCompress:MissingData','Jump in timestamp: Data are not contiguous')
                end
            end
            
            % Read file:
            % Counters:
            % numFramesInFile = number of frames in current file
            % numAllFrames = Frames in files plus the excess frames from previous file
            % numReadFrames = number of frames to read from current data
            % numRemainingFrames = number of frames to be read from current data into excess Frame
            % numStackedFrames = number of frames to process after stacking
            numFramesInFile = dFileStr(ii).bytes/frameSize;
            numFramesToStack = (numFramesInFile + numExcessFrames(1)) - mod((numFramesInFile + numExcessFrames(1)),numAvg);
            numRemainingFrames = numFramesInFile- (numFramesToStack - numExcessFrames(1));
            numStackedFrames = numFramesToStack/numAvg;
            fid1 = fopen([rawdatapath,dFileStr(ii).name],'r');
            cleaner = onCleanup(@()fclose(fid1));
            tmpfile(1:numFramesInFile*frameSize/2) = fread(fid1,numFramesInFile*frameSize/2,'int16');
            clear('cleaner');
            
            % Check number of frames:
            if numFramesInFile == 0
                warning('dicePulseCompress:ZeroFileSize','Skipping empty file %s',dFileStr(ii).name);
                continue
            end
            
           if numFramesToStack == 0
               warning('dicePulseCompress:notEnoughFrames','Not enough frames in %s - Skipping',dFileStr(ii).name);
               continue
           end
                        
            % Communicate with human:
            fprintf('dicePulseCompress: Processing file %d of %d\n',ii-minfNum(batch)+1, maxfNum(batch)-minfNum(batch)+1)
            
            % Loop through channels:
            for channel=1:numChannels
                
                % Read unstacked data:
                if numExcessFrames(channel)>0
                    dataprestack=[excessdata{channel}(:,1:numExcessFrames(channel)),reshape(tmpfile(dataind{channel}(1:frameLen*numFramesInFile))*elephantstoVolts,Nt(channel),numFramesInFile)];
                    if process_metadata
                        metadataprestack=[excessmetadata{channel}(:,1:numExcessFrames(channel)),reshape(tmpfile(metadataind{channel}(1:frameLen*numFramesInFile)),numMetadataSamples,numFramesInFile)];
                    end
                else
                    dataprestack=   reshape(tmpfile(dataind{channel}(1:frameLen*numFramesInFile))*elephantstoVolts,Nt(channel),numFramesInFile);
                    if process_metadata 
                        metadataprestack = reshape(tmpfile(metadataind{channel}(1:frameLen*numFramesInFile)),numMetadataSamples,numFramesInFile);
                    end
                end
                
                % Save excess data to be left after stacking
                if numRemainingFrames>0
                    excessdata{channel}(:,1:numRemainingFrames)=dataprestack(1:Nt(channel),numAvg*numStackedFrames+1:end);
                    if process_metadata
                        excessmetadata{channel}(:,1:numRemainingFrames)=metadataprestack(1:numMetadataSamples,numAvg*numStackedFrames+1:end);
                    end
                end
                numExcessFrames(channel)=numRemainingFrames;
                
                %% Stack Data and metadata
                % Stack data (sum or mean)
                datastack{channel}(1:Nt(channel),1:numStackedFrames) = reshape(mean(reshape(dataprestack(1:Nt(channel),1:numAvg*numStackedFrames),Nt(channel),numAvg,numStackedFrames),2),Nt(channel),numStackedFrames);
                % Stack metadata:
                if process_metadata
                    metadatatmp{channel}(:,numechos(channel):numechos(channel)+numStackedFrames-1,:) = permute(reshape(metadataprestack(:,1:numAvg*numStackedFrames),numMetadataSamples,numAvg,numStackedFrames),[1,3,2]);
                end
                % Remove mean signal from each echo
                datastack{channel}(1:Nt(channel),1:numStackedFrames) = detrend(datastack{channel}(1:Nt(channel),1:numStackedFrames),'constant');
                %% Use Notch Filter if needed to take out specific spurious peaks
                if usenotchfilter==1
                    for ff = 1:numNotch
                        datastack{channel}(1:Nt(channel),1:numStackedFrames) = filtfilt(notchFilterObj{ff},datastack{channel}(1:Nt(channel),1:numStackedFrames));
                    end
                end
                if removespikes==1
                    for rind = 1:size(spikeStartInd,2);
                        datastack{channel}(spikeStartInd(channel,rind):spikeStopInd(channel,rind),1:numStackedFrames)= randn(spikeStopInd(channel,rind)-spikeStartInd(channel,rind)+1, numStackedFrames)*noiseVal;
                    end
                end
                %% demodulate and filter
                if(doCompression ==1)
                    datastack{channel}(1:Nt(channel),1:numStackedFrames) = datastack{channel}(1:Nt(channel),1:numStackedFrames).*iqDemod{channel}(1:Nt(channel),1:numStackedFrames);
                end
                if(uselpf)
                    datastack{channel}(1:Nt(channel),1:numStackedFrames) = filtfilt(dicelpf,1,datastack{channel}(1:Nt(channel),1:numStackedFrames));
                end
                
                % Perform pulse compression:
                if(doCompression ==1)
                    tmpdata{channel}(1:NFFT(channel),1:numStackedFrames) = ifft(fft([zeros(Nref(channel)-1,numStackedFrames);datastack{channel}(1:Nt(channel),1:numStackedFrames)],NFFT(channel)).*ref{channel}(1:NFFT(channel),1:numStackedFrames),NFFT(channel));
                    datastack{channel}(1:truncFFTLen(channel),1:numStackedFrames) = real(tmpdata{channel}(1:truncFFTLen(channel),1:numStackedFrames))+ 1j*imag(tmpdata{channel}(1:truncFFTLen(channel),1:numStackedFrames));
                    % Assign this file's output matrix:
                    radechotmp{channel}(1:Nt(channel),numechos(channel):numechos(channel)+numStackedFrames-1) = datastack{channel}(Nref(channel):truncFFTLen(channel),1:numStackedFrames);
                else
                    radechotmp{channel}(1:Nt(channel),numechos(channel):numechos(channel)+numStackedFrames-1) = datastack{channel}(1:Nt(channel),1:numStackedFrames);
                end
                % Compute utc time vector:
                utctmp{channel}(numechos(channel):numechos(channel)+numStackedFrames-1)= lastTime(channel)+(0:numStackedFrames-1)*1/prfPreSar;
                lastTime(channel) = utctmp{channel}(numechos(channel)+numStackedFrames-1)+1/prfPreSar;
                % Increment number of echos:
                numechos(channel) = numechos(channel) + numStackedFrames;
            end
            
        end
        
        % Loop through channels:
        for channel=1:numChannels
            
            % Create final output variables for this batch:
            if(doCompression ==1)
                radecho = radechotmp{channel}(1:Nt(channel),1:numechos(channel)-1);
                txOn = 20*log10(max(abs(radecho(TxInd-10:TxInd+10,:)),[],1))>txThreshold(processChan(channel));
            else
                radecho = radechotmp{channel}(1:Nt(channel),1:numechos(channel)-1);
                txOn = ones(size(radecho,2))*NaN;
            end
            if(process_metadata == 1)
                metadata = metadatatmp{channel}(:,1:numechos(channel)-1,:);
            end
            echotimeutc = utctmp{channel}(1:numechos(channel)-1);
            filesinbatch= dFileStr(minfNum(batch):maxfNum(batch));
            numexcessFrames = numExcessFrames(channel);
            % Assign this echo type:
            echotype=echotypetmp{channel};
            
            % Compute two-way travel time vector:
            twtt = linspace(0,Nt(channel)/fs,Nt(channel))';
            % set product level
            prodLevel.name = 'level1';
            %If data available, Create output batch name:
            if ~isempty(echotimeutc)
                % Create output batch name:
                outputname = [Campaign,fileFieldSep,FlightNo,fileFieldSep,...
                    datestr(unix2matlabtime(echotimeutc(1)),[outFileDateFormat,...
                    fileFieldSep,outFileTimeFormat]),fileFieldSep,echotype,fileFieldSep,prodLevel.name,fileFieldSep,Sensor,'.mat'];
                % Save output file:
                if ~exist([pulseCompressedDataPath,outputname],'file') || overwriteoutput
                    if(process_metadata == 1)
                        save([pulseCompressedDataPath,outputname],'Campaign','prodLevel','FlightNo','twtt','radecho','echotimeutc','metadata','echotype','filesinbatch','numAvg','numexcessFrames','txOn','pulseCompressedTime');
                    else
                        save([pulseCompressedDataPath,outputname],'Campaign','prodLevel','FlightNo','twtt','radecho','echotimeutc','echotype','filesinbatch','numAvg','numexcessFrames','txOn','pulseCompressedTime');
                    end
                else
                   fprintf('Skipped %s!\n',outputname);
                end
            end
        end
        % Incremement file numbers:
%         minfNum=minfNum+filesperbatch;
%         maxfNum=min(maxfNum+filesperbatch,numDataFiles);
        
    end
    warning('on','all');
    diary off;
    [~,logFileNameOnly,logFileExt] = fileparts(logFile);
    if ~exist(processLogPath,'dir')
        mkdir(processLogPath);
    end
    movefile(logFile,[processLogPath,logFileNameOnly,fileFieldSep,'pulsecompress',datestr(now,'_yyyymmdd_hhMMss'),logFileExt]);
    [~,configFileNameOnly,~] = fileparts(configScript);
    copyfile(configScript,[processLogPath,configFileNameOnly,datestr(pulseCompressedTime,'_yyyymmdd_hhMMss'),'.m.log']);
    diary(logFile);
end
    if localDiary % if diary was locally started - delete it
        if strcmpi(get(0,'Diary'),'on')
            diary off;
            [~,~,logext] = fileparts(logFile);
            if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
                delete(logFile);
            end
        end
    end
fprintf('dicePulseCompress: Done!\n')
toc

catch errMsg
    fprintf('dicePulseCompress: Encountered an error\n');
    fprintf('%s\n',errMsg.message);
    fprintf('%s\n',errMsg.identifier);
    for jj = 1:numel(errMsg.stack)
        fprintf('In Function %s, Line %d\n',errMsg.stack(jj).name,errMsg.stack(jj).line)
    end
    warning('on','all');
    % move the log file
    diary off;
    if exist(logFile,'file') && exist('processLogPath','var')
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        if ~exist(processLogPath,'dir')
            mkdir(processLogPath);
        end
        movefile(logFile,[processLogPath,logFileNameOnly,'_crash',datestr(now,'_yyyymmdd_hhMMss'),logFileExt]);
    end
    if debugEnable
        fprintf('dicePulseCompress: Halting for debugging\n')
        keyboard
    end
end
end
